"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/* eslint-disable no-param-reassign */
/* eslint-disable node/no-extraneous-import */
/* eslint-disable import/no-extraneous-dependencies */
require("reflect-metadata");
const logger_1 = require("@slack/logger");
class TypeORMInstallationStore {
    constructor(options) {
        this.connectionProvider = options.connectionProvider;
        this.connectionCacheEnabled = options.connectionCacheEnabled !== undefined ?
            options.connectionCacheEnabled : true;
        this.connection = options.connection;
        if (this.connectionProvider === undefined && this.connection === undefined) {
            throw new Error('Either connectionProvider or connection is required to initialize this InstallationStore');
        }
        this.clientId = options.clientId;
        this.logger = options.logger || new logger_1.ConsoleLogger();
        this.historicalDataEnabled = options.historicalDataEnabled !== undefined ?
            options.historicalDataEnabled : true;
        this.onStoreInstallation = options.onStoreInstallation !== undefined ?
            options.onStoreInstallation : async (_) => { };
        this.onFetchInstallation = options.onFetchInstallation !== undefined ?
            options.onFetchInstallation : async (_) => { };
        this.onDeleteInstallation = options.onDeleteInstallation !== undefined ?
            options.onDeleteInstallation : async (_) => { };
        this.entityFactory = options.entityFactory;
        this.entityTarget = options.entityTarget;
        this.sortPropertyName = options.sortPropertyName || 'id';
        this.logger.debug(`TypeORMInstallationStore has been initialized (clientId: ${this.clientId}, entityTarget: ${this.entityTarget})`);
    }
    async storeInstallation(i, logger) {
        var _a, _b;
        const enterpriseId = (_a = i.enterprise) === null || _a === void 0 ? void 0 : _a.id;
        const teamId = (_b = i.team) === null || _b === void 0 ? void 0 : _b.id;
        const userId = i.user.id;
        const isEnterpriseInstall = i.isEnterpriseInstall || false;
        const commonLogPart = `(enterprise_id: ${enterpriseId}, team_id: ${teamId}, user_id: ${userId})`;
        logger === null || logger === void 0 ? void 0 : logger.debug(`#storeInstallation starts ${commonLogPart}`);
        let row;
        if (this.historicalDataEnabled) {
            // In this mode, we always insert new row
            row = this.entityFactory();
            row.clientId = this.clientId;
        }
        else {
            const existingRow = await this.findRow({
                teamId,
                enterpriseId,
                userId,
                isEnterpriseInstall,
            });
            if (existingRow !== undefined) {
                row = existingRow;
            }
            else {
                row = this.entityFactory();
                row.clientId = this.clientId;
            }
        }
        await this.setupRow(row, i);
        await this.onStoreInstallation({
            entity: row,
            installation: i,
            logger: this.logger,
        });
        const conn = await this.getConnection();
        await conn.manager.save(row);
        logger === null || logger === void 0 ? void 0 : logger.debug(`#storeInstallation successfully completed ${commonLogPart}`);
    }
    async fetchInstallation(query, logger) {
        const { enterpriseId, teamId, userId, isEnterpriseInstall } = query;
        const commonLogPart = `(enterprise_id: ${enterpriseId}, team_id: ${teamId}, user_id: ${userId})`;
        logger === null || logger === void 0 ? void 0 : logger.debug(`#fetchInstallation starts ${commonLogPart}`);
        const botQuery = {
            enterpriseId,
            teamId,
            isEnterpriseInstall,
        };
        let installation;
        let mergedRow;
        const botRow = await this.findRow(botQuery);
        if (botRow) {
            installation = buildInstallationFromRow(botRow);
        }
        if ((botRow === null || botRow === void 0 ? void 0 : botRow.userId) !== query.userId) {
            botRow === null || botRow === void 0 ? true : delete botRow.userRefreshToken;
            botRow === null || botRow === void 0 ? true : delete botRow.userScopes;
            botRow === null || botRow === void 0 ? true : delete botRow.userToken;
            botRow === null || botRow === void 0 ? true : delete botRow.userTokenExpiresAt;
            const userRow = await this.findRow(query);
            if (userRow) {
                mergedRow = Object.assign(userRow, botRow);
                installation = buildInstallationFromRow(userRow);
            }
        }
        else {
            mergedRow = botRow;
        }
        if (installation) {
            await this.onFetchInstallation({
                query,
                installation,
                entity: mergedRow,
                logger: this.logger,
            });
            return installation;
        }
        logger === null || logger === void 0 ? void 0 : logger.debug(`#fetchInstallation didn't return any installation data ${commonLogPart}`);
        throw new Error(`No installation data found ${commonLogPart}`);
    }
    async deleteInstallation(query, logger) {
        const { enterpriseId, teamId, userId, isEnterpriseInstall } = query;
        const commonLogPart = `(enterprise_id: ${enterpriseId}, team_id: ${teamId}, user_id: ${userId})`;
        logger === null || logger === void 0 ? void 0 : logger.debug(`#deleteInstallation starts ${commonLogPart}`);
        await this.onDeleteInstallation({
            query,
            logger: this.logger,
        });
        const conn = await this.getConnection();
        await conn.manager
            .getRepository(this.entityTarget)
            .createQueryBuilder()
            .delete()
            .where([
            (this.clientId ? 'clientId = :clientId' : 'clientId is null'),
            (enterpriseId ? 'enterpriseId = :enterpriseId' : undefined),
            (isEnterpriseInstall ? 'teamId is null' : undefined),
            (!isEnterpriseInstall && teamId ? 'teamId = :teamId' : undefined),
            (userId ? 'userId = :userId' : undefined),
        ].filter((a) => a !== undefined).join(' AND '))
            .setParameters({ enterpriseId, teamId, userId, clientId: this.clientId })
            .execute();
        logger === null || logger === void 0 ? void 0 : logger.debug(`#deleteInstallation completed ${commonLogPart}`);
    }
    async close() {
        if (this.connection !== undefined && this.connection.isConnected) {
            await this.connection.close();
        }
    }
    // ------------------------------------------
    // private methods
    // ------------------------------------------
    async getConnection() {
        if (this.connection !== undefined) {
            if (this.connection.isConnected) {
                return this.connection;
            }
            await this.connection.close();
        }
        if (this.connectionProvider !== undefined) {
            return this.connectionProvider().then((conn) => {
                if (this.connectionCacheEnabled) {
                    this.connection = conn;
                }
                return conn;
            });
        }
        throw new Error('Unexpectedly both connection and connectionProvider are missing!');
    }
    async setupRow(entity, i) {
        var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k, _l, _m, _o, _p, _q, _r, _s, _t;
        const bothClientIdAbsent = (entity.clientId === null || entity.clientId === undefined) &&
            (this.clientId === undefined || this.clientId === null);
        if (!bothClientIdAbsent && entity.clientId !== this.clientId) {
            throw new Error(`Unexpected installation update (client_id: ${entity.clientId}) by a differnt app (client_id: ${this.clientId}) detected`);
        }
        entity.clientId = this.clientId;
        entity.appId = i.appId;
        entity.enterpriseId = (_a = i.enterprise) === null || _a === void 0 ? void 0 : _a.id;
        entity.enterpriseName = (_b = i.enterprise) === null || _b === void 0 ? void 0 : _b.name;
        entity.enterpriseUrl = i.enterpriseUrl;
        entity.teamId = (_c = i.team) === null || _c === void 0 ? void 0 : _c.id;
        entity.teamName = (_d = i.team) === null || _d === void 0 ? void 0 : _d.name;
        entity.botToken = (_e = i.bot) === null || _e === void 0 ? void 0 : _e.token;
        entity.botId = (_f = i.bot) === null || _f === void 0 ? void 0 : _f.id;
        entity.botUserId = (_g = i.bot) === null || _g === void 0 ? void 0 : _g.userId;
        entity.botScopes = (_j = (_h = i.bot) === null || _h === void 0 ? void 0 : _h.scopes) === null || _j === void 0 ? void 0 : _j.join(',');
        if (entity.botRefreshToken !== ((_k = i.bot) === null || _k === void 0 ? void 0 : _k.refreshToken)) {
            entity.botRefreshToken = (_l = i.bot) === null || _l === void 0 ? void 0 : _l.refreshToken;
            entity.botTokenExpiresAt = ((_m = i.bot) === null || _m === void 0 ? void 0 : _m.expiresAt) ?
                new Date(i.bot.expiresAt * 1000) :
                undefined;
        }
        entity.userId = i.user.id;
        entity.userToken = i.user.token;
        entity.userScopes = (_o = i.user.scopes) === null || _o === void 0 ? void 0 : _o.join(',');
        if (entity.userRefreshToken !== i.user.refreshToken) {
            entity.userRefreshToken = i.user.refreshToken;
            entity.userTokenExpiresAt = ((_p = i.user) === null || _p === void 0 ? void 0 : _p.expiresAt) ?
                new Date(i.user.expiresAt * 1000) :
                undefined;
        }
        entity.incomingWebhookUrl = (_q = i.incomingWebhook) === null || _q === void 0 ? void 0 : _q.url;
        entity.incomingWebhookChannel = (_r = i.incomingWebhook) === null || _r === void 0 ? void 0 : _r.channel;
        entity.incomingWebhookChannelId = (_s = i.incomingWebhook) === null || _s === void 0 ? void 0 : _s.channelId;
        entity.incomingWebhookConfigurationUrl = (_t = i.incomingWebhook) === null || _t === void 0 ? void 0 : _t.configurationUrl;
        entity.isEnterpriseInstall = i.isEnterpriseInstall;
        // TODO: need to fix on the @slack/oauth side
        entity.tokenType = i.tokenType === undefined ? 'bot' : i.tokenType;
        if (entity.installedAt === undefined) {
            entity.installedAt = new Date();
        }
    }
    async findRow(query) {
        const { enterpriseId, teamId, isEnterpriseInstall, userId } = query;
        const alias = 'i';
        const conn = await this.getConnection();
        const selectQuery = conn.manager
            .getRepository(this.entityTarget)
            .createQueryBuilder(alias)
            .where(
        // Note that we don't need to worry about NamingStrategy compatibility here
        // TypeORM's internal where clause parser converts the property names such as clientId
        // into the corresponding column names. If you go with snake_case NamingStrategy,
        // the following where clause will use the naming conventions in SQL statements.
        [
            (this.clientId ? `${alias}.clientId = :clientId` : `${alias}.clientId is null`),
            (enterpriseId ? `${alias}.enterpriseId = :enterpriseId` : undefined),
            (isEnterpriseInstall ? `${alias}.teamId is null` : undefined),
            (!isEnterpriseInstall && teamId ? `${alias}.teamId = :teamId` : undefined),
            (userId ? `${alias}.userId = :userId` : undefined),
        ].filter((a) => a !== undefined).join(' AND '))
            .setParameters({ enterpriseId, teamId, userId, clientId: this.clientId });
        if (this.historicalDataEnabled) {
            return selectQuery.orderBy(`${alias}.${this.sortPropertyName}`, 'DESC')
                .limit(1)
                .getOne();
        }
        return selectQuery.orderBy(`${alias}.${this.sortPropertyName}`, 'DESC').getOne();
    }
}
exports.default = TypeORMInstallationStore;
function buildInstallationFromRow(row) {
    var _a, _b;
    return {
        team: row.teamId ?
            {
                id: row.teamId,
                name: row.teamName,
            } :
            undefined,
        enterprise: row.enterpriseId ?
            {
                id: row.enterpriseId,
                name: row.enterpriseName,
            } :
            undefined,
        enterpriseUrl: row.enterpriseUrl,
        user: {
            // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
            id: row.userId,
            token: row.userToken,
            refreshToken: row.userRefreshToken,
            expiresAt: row.userTokenExpiresAt ? Math.floor(row.userTokenExpiresAt.getTime() / 1000) : undefined,
            scopes: (_a = row.userScopes) === null || _a === void 0 ? void 0 : _a.split(','),
        },
        bot: row.botId && row.botUserId && row.botToken ?
            {
                id: row.botId,
                userId: row.botUserId,
                token: row.botToken,
                refreshToken: row.botRefreshToken,
                expiresAt: row.botTokenExpiresAt ? Math.floor(row.botTokenExpiresAt.getTime() / 1000) : undefined,
                scopes: ((_b = row.botScopes) === null || _b === void 0 ? void 0 : _b.split(',')) || [],
            } :
            undefined,
        incomingWebhook: row.incomingWebhookUrl ?
            {
                url: row.incomingWebhookUrl,
                channel: row.incomingWebhookChannel,
                channelId: row.incomingWebhookChannelId,
                configurationUrl: row.incomingWebhookConfigurationUrl,
            } :
            undefined,
        appId: row.appId,
        // TODO: `as 'bot'` is a workaround; user type is not yet supported in TS
        tokenType: row.tokenType === undefined ? 'bot' : row.tokenType,
        isEnterpriseInstall: row.isEnterpriseInstall,
        authVersion: 'v2', // This module does not support v1 installations
    };
}
//# sourceMappingURL=TypeORMInstallationStore.js.map